@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C)2008 Sourcefire, Inc.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Author: aCaB <acab@clamav.net>
  *
@@ -29,6 +29,8 @@
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
+#include <signal.h>
+#include <pthread.h>
 #ifdef USE_SYSLOG
 #include <syslog.h>
 #endif
@@ -48,15 +50,57 @@
 #include "whitelist.h"
 
 struct smfiDesc descr;
+struct optstruct *opts;
+
+static void milter_exit(int sig) {
+    const struct optstruct *opt;
+
+    logg("*clamav-milter: milter_exit, signal %d\n", sig);
+
+#ifndef _WIN32
+    if((opt = optget(opts, "MilterSocket"))) {
+        if(unlink(opt->strarg) == -1)
+            logg("!Can't unlink the socket file %s\n", opt->strarg);
+        else
+            logg("Socket file removed.\n");
+    }
+#endif
+
+    logg("clamav-milter: stopped\n");
+
+    optfree(opts);
+
+    logg_close();
+    cpool_free();
+    localnets_free();
+    whitelist_free();
+}
 
 int main(int argc, char **argv) {
     char *my_socket, *pt;
     const struct optstruct *opt;
-    struct optstruct *opts;
     time_t currtime;
     mode_t umsk;
     int ret;
 
+    sigset_t sigset;
+    struct sigaction act;
+
+    sigfillset(&sigset);
+    sigdelset(&sigset, SIGUSR1);
+    sigdelset(&sigset, SIGFPE);
+    sigdelset(&sigset, SIGILL);
+    sigdelset(&sigset, SIGSEGV);
+#ifdef SIGBUS
+    sigdelset(&sigset, SIGBUS);
+#endif
+    pthread_sigmask(SIG_SETMASK, &sigset, NULL);
+    memset(&act, 0, sizeof(struct sigaction));
+    act.sa_handler = milter_exit;
+    sigfillset(&(act.sa_mask));
+    sigaction(SIGUSR1, &act, NULL);
+    sigaction(SIGSEGV, &act, NULL);
+
     cl_initialize_crypto();
 
     memset(&descr, 0, sizeof(struct smfiDesc));
@@ -78,10 +122,17 @@ int main(int argc, char **argv) {
     }
 
     if(optget(opts, "help")->enabled) {
-	printf("Usage: %s [-c <config-file>]\n\n", argv[0]);
+    printf("\n");
+    printf("                       Clam AntiVirus: Milter Mail Scanner %s\n", get_version());
+    printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2019 Cisco Systems, Inc.\n");
+    printf("\n");
+	printf("    %s [-c <config-file>]\n\n", argv[0]);
+    printf("\n");
 	printf("    --help                   -h       Show this help\n");
-	printf("    --version                -V       Show version and exit\n");
-	printf("    --config-file <file>     -c       Read configuration from file\n\n");
+	printf("    --version                -V       Show version\n");
+	printf("    --config-file <file>     -c       Read configuration from file\n");
+    printf("\n");
 	optfree(opts);
 	return 0;
     }
@@ -254,28 +305,19 @@ int main(int argc, char **argv) {
 	    return 1;
 	}
 
-	if(optget(opts, "AllowSupplementaryGroups")->enabled) {
 #ifdef HAVE_INITGROUPS
-	    if(initgroups(opt->strarg, user->pw_gid)) {
-		fprintf(stderr, "ERROR: initgroups() failed.\n");
-		optfree(opts);
-		return 1;
-	    }
-#else
-	    mprintf("!AllowSupplementaryGroups: initgroups() is not available, please disable AllowSupplementaryGroups\n");
+	if(initgroups(opt->strarg, user->pw_gid)) {
+	    fprintf(stderr, "ERROR: initgroups() failed.\n");
 	    optfree(opts);
 	    return 1;
-#endif
-	} else {
-#ifdef HAVE_SETGROUPS
-	    if(setgroups(1, &user->pw_gid)) {
-		fprintf(stderr, "ERROR: setgroups() failed.\n");
-		optfree(opts);
-		return 1;
-	    }
-#endif
 	}
-
+#elif HAVE_SETGROUPS
+	if(setgroups(1, &user->pw_gid)) {
+	    fprintf(stderr, "ERROR: setgroups() failed.\n");
+	    optfree(opts);
+	    return 1;
+	}
+#endif
 	if(setgid(user->pw_gid)) {
 	    fprintf(stderr, "ERROR: setgid(%d) failed.\n", (int) user->pw_gid);
 	    optfree(opts);
@@ -403,16 +445,7 @@ int main(int argc, char **argv) {
 	umask(old_umask);
     }
 
-    ret = smfi_main();
-
-    optfree(opts);
-
-    logg_close();
-    cpool_free();
-    localnets_free();
-    whitelist_free();
-
-    return ret;
+    return smfi_main();
 }
 
 /*
