@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -73,7 +73,7 @@
 #include "scanner.h"
 
 short debug_mode = 0, logok = 0;
-short foreground = 0;
+short foreground = -1;
 char hostid[37];
 
 char *get_hostid(void *cbdata);
@@ -82,14 +82,19 @@ int is_valid_hostid(void);
 static void help(void)
 {
     printf("\n");
-    printf("                      Clam AntiVirus Daemon %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2007-2015 Cisco Systems, Inc.\n\n");
-
-    printf("    --help                   -h             Show this help.\n");
-    printf("    --version                -V             Show version number.\n");
-    printf("    --debug                                 Enable debug mode.\n");
-    printf("    --config-file=FILE       -c FILE        Read configuration from FILE.\n\n");
+    printf("                      Clam AntiVirus: Daemon %s\n", get_version());
+    printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2019 Cisco Systems, Inc.\n");
+    printf("\n");
+    printf("    clamd [options]\n");
+    printf("\n");
+    printf("    --help                   -h             Show this help\n");
+    printf("    --version                -V             Show version number\n");
+    printf("    --debug                                 Enable debug mode\n");
+    printf("    --config-file=FILE       -c FILE        Read configuration from FILE\n");
+    printf("\n");
+    printf("Pass in - as the filename for stdin.\n");
+    printf("\n");
 }
 
 static struct optstruct *opts;
@@ -125,7 +130,9 @@ int main(int argc, char **argv)
     int *lsockets=NULL;
     unsigned int nlsockets = 0;
     unsigned int dboptions = 0;
-    unsigned int i;
+    unsigned int i;	
+    int j;
+    int num_fd;
 #ifdef C_LINUX
     STATBUF sb;
 #endif
@@ -161,6 +168,30 @@ int main(int argc, char **argv)
         debug_mode = 1;
     }
 
+    /* check foreground option from command line to override config file */
+    for(j = 0; j < argc; j += 1)
+    {
+        if ((memcmp(argv[j], "--foreground", 12) == 0) || (memcmp(argv[j], "-F", 2) == 0))
+        {
+            /* found */
+            break;
+        }
+    }
+
+    if (j < argc)
+    {
+        if(optget(opts, "Foreground")->enabled)
+        {
+            foreground = 1;
+        }
+        else
+        {
+            foreground = 0;
+        }
+    }
+
+    num_fd = sd_listen_fds(0);
+
     /* parse the config file */
     cfgfile = optget(opts, "config-file")->strarg;
     pt = strdup(cfgfile);
@@ -190,27 +221,19 @@ int main(int argc, char **argv)
             return 1;
         }
 
-        if(optget(opts, "AllowSupplementaryGroups")->enabled) {
 #ifdef HAVE_INITGROUPS
-            if(initgroups(opt->strarg, user->pw_gid)) {
-                fprintf(stderr, "ERROR: initgroups() failed.\n");
+	if(initgroups(opt->strarg, user->pw_gid)) {
+		fprintf(stderr, "ERROR: initgroups() failed.\n");
+                optfree(opts);
+		return 1;
+	}
+#elif HAVE_SETGROUPS
+	if(setgroups(1, &user->pw_gid)) {
+		fprintf(stderr, "ERROR: setgroups() failed.\n");
                 optfree(opts);
                 return 1;
-            }
-#else
-            mprintf("!AllowSupplementaryGroups: initgroups() is not available, please disable AllowSupplementaryGroups in %s\n", cfgfile);
-            optfree(opts);
-            return 1;
+	}
 #endif
-        } else {
-#ifdef HAVE_SETGROUPS
-            if(setgroups(1, &user->pw_gid)) {
-                fprintf(stderr, "ERROR: setgroups() failed.\n");
-                optfree(opts);
-                return 1;
-            }
-#endif
-        }
 
         if(setgid(user->pw_gid)) {
             fprintf(stderr, "ERROR: setgid(%d) failed.\n", (int) user->pw_gid);
@@ -300,7 +323,9 @@ int main(int argc, char **argv)
         if(optget(opts, "LocalSocket")->enabled)
             localsock = 1;
 
-        if(!tcpsock && !localsock) {
+        logg("#Received %d file descriptor(s) from systemd.\n", num_fd);
+
+        if(!tcpsock && !localsock && num_fd == 0) {
             logg("!Please define server type (local and/or TCP).\n");
             ret = 1;
             break;
@@ -333,7 +358,7 @@ int main(int argc, char **argv)
 
 
         if(logg_size)
-            logg("#Log file size limited to %u bytes.\n", logg_size);
+            logg("#Log file size limited to %lld bytes.\n", (long long int)logg_size);
         else
             logg("#Log file size limit disabled.\n");
 
@@ -345,6 +370,15 @@ int main(int argc, char **argv)
             break;
         }
 
+        /* TODO: Re-enable OnAccessExtraScanning once the thread resource consumption issue is resolved. */
+        if(optget(opts, "OnAccessExtraScanning")->enabled) {
+            logg("*ScanOnAccess: OnAccessExtraScanning was requested, but has "
+                 "been disabled due to a known issue with thread resource "
+                 "cleanup. The OnAccessExtraScanning feature will be "
+                 "re-enabled in a future release when the issue is resolved. "
+                 "For details, see: https://bugzilla.clamav.net/show_bug.cgi?id=12048\n");
+        }
+
         if(!(engine = cl_engine_new())) {
             logg("!Can't initialize antivirus engine\n");
             ret = 1;
@@ -437,42 +471,6 @@ int main(int argc, char **argv)
             logg("#Not loading PUA signatures.\n");
         }
 
-        if (optget(opts, "StatsEnabled")->enabled) {
-            cl_engine_stats_enable(engine);
-        }
-
-        if (optget(opts, "StatsPEDisabled")->enabled) {
-            cl_engine_set_num(engine, CL_ENGINE_DISABLE_PE_STATS, 1);
-        }
-
-        if (optget(opts, "StatsTimeout")->enabled) {
-            cl_engine_set_num(engine, CL_ENGINE_STATS_TIMEOUT, optget(opts, "StatsTimeout")->numarg);
-        }
-
-        if (optget(opts, "StatsHostID")->enabled) {
-            char *p = optget(opts, "StatsHostID")->strarg;
-
-            if (strcmp(p, "default")) {
-                if (!strcmp(p, "none")) {
-                    cl_engine_set_clcb_stats_get_hostid(engine, NULL);
-                } else if (!strcmp(p, "anonymous")) {
-                    strcpy(hostid, STATS_ANON_UUID);
-                } else {
-                    if (strlen(p) > 36) {
-                        logg("!Invalid HostID\n");
-                        cl_engine_set_clcb_stats_submit(engine, NULL);
-                        cl_engine_free(engine);
-                        ret = 1;
-                        break;
-                    }
-
-                    strcpy(hostid, p);
-                }
-
-                cl_engine_set_clcb_stats_get_hostid(engine, get_hostid);
-            }
-        }
-
         if(optget(opts, "OfficialDatabaseOnly")->enabled) {
             dboptions |= CL_DB_OFFICIAL_ONLY;
             logg("#Only loading official signatures.\n");
@@ -580,17 +578,36 @@ int main(int argc, char **argv)
         }
 
         if (optget(opts, "DisableCertCheck")->enabled)
-            engine->dconf->pe |= PE_CONF_DISABLECERT;
+            cl_engine_set_num(engine, CL_ENGINE_DISABLE_PE_CERTS, 1);
 
         logg("#Loaded %u signatures.\n", sigs);
 
+        /* pcre engine limits - required for cl_engine_compile */
+        if((opt = optget(opts, "PCREMatchLimit"))->active) {
+            if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
+                logg("!cli_engine_set_num(PCREMatchLimit) failed: %s\n", cl_strerror(ret));
+                cl_engine_free(engine);
+                return 1;
+            }
+        }
+
+        if((opt = optget(opts, "PCRERecMatchLimit"))->active) {
+            if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
+                logg("!cli_engine_set_num(PCRERecMatchLimit) failed: %s\n", cl_strerror(ret));
+                cl_engine_free(engine);
+                return 1;
+            }
+        }
+
         if((ret = cl_engine_compile(engine)) != 0) {
             logg("!Database initialization error: %s\n", cl_strerror(ret));
             ret = 1;
             break;
         }
 
-        if(tcpsock) {
+        if(tcpsock || num_fd > 0) {
+            int *t;
+
             opt = optget(opts, "TCPAddr");
             if (opt->enabled) {
                 int breakout = 0;
@@ -617,7 +634,7 @@ int main(int argc, char **argv)
             }
         }
 #ifndef _WIN32
-        if(localsock) {
+        if(localsock && num_fd == 0) {
             int *t;
             mode_t sock_mode, umsk = umask(0777); /* socket is created with 000 to avoid races */
 
@@ -679,8 +696,43 @@ int main(int argc, char **argv)
             nlsockets++;
         }
 
+        /* check for local sockets passed by systemd */
+        if (num_fd > 0)
+        {
+            int *t;
+            t = realloc(lsockets, sizeof(int) * (nlsockets + 1));
+            if (!(t)) {
+                ret = 1;
+                break;
+            }
+            lsockets = t;
+
+            lsockets[nlsockets] = localserver(opts);
+            if (lsockets[nlsockets] == -1)
+            {
+                ret = 1;
+                break;
+            }
+            else if (lsockets[nlsockets] > 0)
+            {
+                nlsockets++;
+            }
+        }
+
         /* fork into background */
-        if(!optget(opts, "Foreground")->enabled) {
+        if (foreground == -1)
+        {
+            if (optget(opts, "Foreground")->enabled)
+            {
+                foreground = 1;
+            }
+            else
+            {
+                foreground = 0;
+            }
+        }
+        if(foreground == 0)
+        {
 #ifdef C_BSD	    
             /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
             for(ret=0;(unsigned int)ret<nlsockets;ret++) {
@@ -714,8 +766,6 @@ int main(int argc, char **argv)
                 if(chdir("/") == -1)
                     logg("^Can't change current working directory to root\n");
 
-        } else {
-            foreground = 1;
         }
 #endif
 
@@ -729,22 +779,24 @@ int main(int argc, char **argv)
 
     } while (0);
 
-    logg("*Closing the main socket%s.\n", (nlsockets > 1) ? "s" : "");
-
-    for (i = 0; i < nlsockets; i++) {
-        closesocket(lsockets[i]);
-    }
+    if (num_fd == 0)
+    {
+        logg("*Closing the main socket%s.\n", (nlsockets > 1) ? "s" : "");
 
+        for (i = 0; i < nlsockets; i++) {
+            closesocket(lsockets[i]);
+        }
 #ifndef _WIN32
-    if(nlsockets && localsock) {
-        opt = optget(opts, "LocalSocket");
+        if(nlsockets && localsock) {
+            opt = optget(opts, "LocalSocket");
 
-        if(unlink(opt->strarg) == -1)
-            logg("!Can't unlink the socket file %s\n", opt->strarg);
-        else
-            logg("Socket file removed.\n");
-    }
+            if(unlink(opt->strarg) == -1)
+                logg("!Can't unlink the socket file %s\n", opt->strarg);
+            else
+                logg("Socket file removed.\n");
+        }
 #endif
+    }
 
     free(lsockets);
 
