@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2009 Sourcefire, Inc.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
  *
@@ -23,8 +24,10 @@
 #endif
 
 #if defined(C_SOLARIS)
+#ifndef __EXTENSIONS__
 #define __EXTENSIONS__
 #endif
+#endif
 
 /* must be first because it may define _XOPEN_SOURCE */
 #include "shared/fdpassing.h"
@@ -45,6 +48,7 @@
 #ifndef _WIN32
 #include <arpa/inet.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netdb.h>
 #endif
 
@@ -82,8 +86,10 @@ int dconnect() {
         if ((sockd = socket(AF_UNIX, SOCK_STREAM, 0)) >= 0) {
             if (connect(sockd, (struct sockaddr *)&nixsock, sizeof(nixsock)) == 0)
                 return sockd;
-            else
+            else {
+                logg("!Could not connect to clamd on LocalSocket %s: %s\n", opt->strarg, strerror(errno));
                 close(sockd);
+            }
         }
     }
 #endif
@@ -92,38 +98,39 @@ int dconnect() {
 
     opt = optget(clamdopts, "TCPAddr");
     while (opt) {
-        ipaddr = NULL;
-        if (opt->strarg)
-            ipaddr = (!strcmp(opt->strarg, "any") ? NULL : opt->strarg);
+        if (opt->enabled) {
+            ipaddr = NULL;
+            if (opt->strarg)
+                ipaddr = (!strcmp(opt->strarg, "any") ? NULL : opt->strarg);
 
-        memset(&hints, 0x00, sizeof(struct addrinfo));
-        hints.ai_family = AF_UNSPEC;
-        hints.ai_socktype = SOCK_STREAM;
+            memset(&hints, 0x00, sizeof(struct addrinfo));
+            hints.ai_family = AF_UNSPEC;
+            hints.ai_socktype = SOCK_STREAM;
 
-        if ((res = getaddrinfo(ipaddr, port, &hints, &info))) {
-            logg("!Could not lookup %s: %s\n", ipaddr ? ipaddr : "", gai_strerror(res));
-            opt = opt->nextarg;
-            continue;
-        }
-
-        for (p = info; p != NULL; p = p->ai_next) {
-            if((sockd = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) < 0) {
-                logg("!Can't create the socket: %s\n", strerror(errno));
+            if ((res = getaddrinfo(ipaddr, port, &hints, &info))) {
+                logg("!Could not lookup %s: %s\n", ipaddr ? ipaddr : "", gai_strerror(res));
+                opt = opt->nextarg;
                 continue;
             }
 
-            if(connect(sockd, p->ai_addr, p->ai_addrlen) < 0) {
-                logg("!Could not connect to clamd on %s: %s\n", opt->strarg, strerror(errno));
-                closesocket(sockd);
-                continue;
+            for (p = info; p != NULL; p = p->ai_next) {
+                if((sockd = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) < 0) {
+                    logg("!Can't create the socket: %s\n", strerror(errno));
+                    continue;
+                }
+
+                if(connect(sockd, p->ai_addr, p->ai_addrlen) < 0) {
+                    logg("!Could not connect to clamd on %s: %s\n", opt->strarg, strerror(errno));
+                    closesocket(sockd);
+                    continue;
+                }
+
+                freeaddrinfo(info);
+                return sockd;
             }
 
             freeaddrinfo(info);
-            return sockd;
         }
-
-        freeaddrinfo(info);
-
         opt = opt->nextarg;
     }
 
@@ -224,7 +231,8 @@ static int chkpath(const char *path)
    if((opt = optget(clamdopts, "ExcludePath"))->enabled) {
 	while(opt) {
 	    if(match_regex(path, opt->strarg) == 1) {
-		logg("~%s: Excluded\n", path);
+                if (printinfected != 1)
+                    logg("~%s: Excluded\n", path);
 		return 1;
 	    }
 	    opt = opt->nextarg;
@@ -317,9 +325,18 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
 		    logg("Failed to parse reply: \"%s\"\n", bol);
 		return -1;
 	    } else if(!memcmp(eol - 7, " FOUND", 6)) {
+                static char last_filename[PATH_MAX+1] = {'\0'};
 		*(eol - 7) = 0;
 		*printok = 0;
-		infected++;
+                if (scantype != ALLMATCH) {
+                    infected++;
+                } else {
+                    if (filename != NULL && strcmp(filename, last_filename)) {
+                        infected++;
+                        strncpy(last_filename, filename, PATH_MAX);
+                        last_filename[PATH_MAX] = '\0';
+                    }
+                }
 		if(filename) {
 		    if(scantype >= STREAM) {
 			logg("~%s%s FOUND\n", filename, colon);
