@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -133,7 +133,7 @@ struct metachain {
 
 struct clamscan_cb_data {
     struct metachain * chain;
-    char * filename;
+    const char * filename;
 };
 
 static cl_error_t pre(int fd, const char *type, void *context)
@@ -183,7 +183,7 @@ static int print_chain(struct metachain *c, char *str, size_t len)
 static cl_error_t post(int fd, int result, const char *virname, void *context)
 {
     struct clamscan_cb_data *d = context;
-    struct metachain *c;
+    struct metachain *c = NULL;
     char str[128];
 
     UNUSEDPARAM(fd);
@@ -277,7 +277,7 @@ static cl_error_t meta(const char* container_type, unsigned long fsize_container
 static void clamscan_virus_found_cb(int fd, const char *virname, void *context)
 {
     struct clamscan_cb_data *data = (struct clamscan_cb_data *)context;
-    char * filename;
+    const char * filename;
 
     if (data == NULL)
         return;
@@ -369,6 +369,12 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
         chain.chains = malloc(sizeof(char **));
         if (chain.chains) {
             chain.chains[0] = strdup(filename);
+            if (!chain.chains[0]) {
+                free(chain.chains);
+                logg("Unable to allocate memory in scanfile()\n");
+                info.errors++;
+                return;
+            }
             chain.nchains = 1;
         }
     }
@@ -389,14 +395,11 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
                 char str[128];
                 int toolong = print_chain(&chain, str, sizeof(str));
 
-                logg("~%s%s!(%u)%s: %s FOUND\n", str, toolong ? "..." : "", chain.lastvir-1, chain.chains[chain.nchains-1], virname);
+                logg("~%s%s!(%llu)%s: %s FOUND\n", str, toolong ? "..." : "", (long long unsigned)(chain.lastvir-1), chain.chains[chain.nchains-1], virname);
             } else if (chain.lastvir) {
-                logg("~%s!(%u): %s FOUND\n", filename, chain.lastvir-1, virname);
+                logg("~%s!(%llu): %s FOUND\n", filename, (long long unsigned)(chain.lastvir-1), virname);
             }
         }
-        if (!(options & CL_SCAN_ALLMATCHES))
-            logg("~%s: %s FOUND\n", filename, virname);
-
         info.files++;
         info.ifiles++;
 
@@ -590,9 +593,6 @@ static int scanstdin(const struct cl_engine *engine, const struct optstruct *opt
     data.filename = "stdin";
     data.chain = NULL;
     if((ret = cl_scanfile_callback(file, &virname, &info.blocks, engine, options, &data)) == CL_VIRUS) {
-        if (!(options & CL_SCAN_ALLMATCHES))
-            logg("stdin: %s FOUND\n", virname);
-
         info.ifiles++;
 
         if(bell)
@@ -670,45 +670,11 @@ int scanmanager(const struct optstruct *opts)
         return 2;
     }
 
+    cl_engine_set_clcb_virus_found(engine, clamscan_virus_found_cb);
+    
     if (optget(opts, "disable-cache")->enabled)
         cl_engine_set_num(engine, CL_ENGINE_DISABLE_CACHE, 1);
 
-    if (optget(opts, "disable-pe-stats")->enabled) {
-        cl_engine_set_num(engine, CL_ENGINE_DISABLE_PE_STATS, 1);
-    }
-
-    if (optget(opts, "enable-stats")->enabled) {
-        cl_engine_stats_enable(engine);
-    }
-
-    if (optget(opts, "stats-timeout")->enabled) {
-        cl_engine_set_num(engine, CL_ENGINE_STATS_TIMEOUT, optget(opts, "StatsTimeout")->numarg);
-    }
-
-    if (optget(opts, "stats-host-id")->enabled) {
-        char *p = optget(opts, "stats-host-id")->strarg;
-
-        if (strcmp(p, "default")) {
-            if (!strcmp(p, "none")) {
-                cl_engine_set_clcb_stats_get_hostid(engine, NULL);
-            } else if (!strcmp(p, "anonymous")) {
-                strcpy(hostid, STATS_ANON_UUID);
-            } else {
-                if (strlen(p) > 36) {
-                    logg("!Invalid HostID\n");
-
-                    cl_engine_set_clcb_stats_submit(engine, NULL);
-                    cl_engine_free(engine);
-                    return 2;
-                }
-
-                strcpy(hostid, p);
-            }
-
-            cl_engine_set_clcb_stats_get_hostid(engine, get_hostid);
-        }
-    }
-
     if(optget(opts, "detect-pua")->enabled) {
         dboptions |= CL_DB_PUA;
         if((opt = optget(opts, "exclude-pua"))->enabled) {
@@ -792,6 +758,12 @@ int scanmanager(const struct optstruct *opts)
     if((opt = optget(opts,"bytecode-timeout"))->enabled)
         cl_engine_set_num(engine, CL_ENGINE_BYTECODE_TIMEOUT, opt->numarg);
 
+    if (optget(opts, "nocerts")->enabled)
+        cl_engine_set_num(engine, CL_ENGINE_DISABLE_PE_CERTS, 1);
+
+    if (optget(opts, "dumpcerts")->enabled)
+        cl_engine_set_num(engine, CL_ENGINE_PE_DUMPCERTS, 1);
+
     if((opt = optget(opts,"bytecode-mode"))->enabled) {
         enum bytecode_mode mode;
 
@@ -819,6 +791,19 @@ int scanmanager(const struct optstruct *opts)
         }
     }
 
+    /* JSON check to prevent engine loading if specified without libjson-c  */
+#if HAVE_JSON
+    if (optget(opts, "gen-json")->enabled)
+        options |= CL_SCAN_FILE_PROPERTIES;
+#else
+    if (optget(opts, "gen-json")->enabled) {
+        logg("!Can't generate json (gen-json). libjson-c dev library was missing or misconfigured when ClamAV was built.\n");
+
+        cl_engine_free(engine);
+        return 2;
+    }
+#endif
+
     if((opt = optget(opts, "tempdir"))->enabled) {
         if((ret = cl_engine_set_str(engine, CL_ENGINE_TMPDIR, opt->strarg))) {
             logg("!cli_engine_set_str(CL_ENGINE_TMPDIR) failed: %s\n", cl_strerror(ret));
@@ -853,6 +838,23 @@ int scanmanager(const struct optstruct *opts)
         free(dbdir);
     }
 
+    /* pcre engine limits - required for cl_engine_compile */
+    if ((opt = optget(opts, "pcre-match-limit"))->active) {
+        if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
+            logg("!cli_engine_set_num(CL_ENGINE_PCRE_MATCH_LIMIT) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 2;
+        }
+    }
+
+    if ((opt = optget(opts, "pcre-recmatch-limit"))->active) {
+        if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
+            logg("!cli_engine_set_num(CL_ENGINE_PCRE_RECMATCH_LIMIT) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 2;
+        }
+    }
+
     if((ret = cl_engine_compile(engine)) != 0) {
         logg("!Database initialization error: %s\n", cl_strerror(ret));;
 
@@ -866,12 +868,6 @@ int scanmanager(const struct optstruct *opts)
         cl_engine_set_clcb_post_scan(engine, post);
     }
 
-    if (optget(opts, "nocerts")->enabled)
-        engine->dconf->pe |= PE_CONF_DISABLECERT;
-
-    if (optget(opts, "dumpcerts")->enabled)
-        engine->dconf->pe |= PE_CONF_DUMPCERT;
-
     /* set limits */
 
     if((opt = optget(opts, "max-scansize"))->active) {
@@ -986,6 +982,15 @@ int scanmanager(const struct optstruct *opts)
         }
     }
 
+    if((opt = optget(opts, "max-rechwp3"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_RECHWP3, opt->numarg))) {
+            logg("!cli_engine_set_num(CL_ENGINE_MAX_RECHWP3) failed: %s\n", cl_strerror(ret));
+
+            cl_engine_free(engine);
+            return 2;
+        }
+    }
+
     if ((opt = optget(opts, "timelimit"))->active) {
         if ((ret = cl_engine_set_num(engine, CL_ENGINE_TIME_LIMIT, opt->numarg))) {
             logg("!cli_engine_set_num(CL_ENGINE_TIME_LIMIT) failed: %s\n", cl_strerror(ret));
@@ -995,22 +1000,6 @@ int scanmanager(const struct optstruct *opts)
         }
     }
 
-    if ((opt = optget(opts, "pcre-match-limit"))->active) {
-        if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
-            logg("!cli_engine_set_num(CL_ENGINE_PCRE_MATCH_LIMIT) failed: %s\n", cl_strerror(ret));
-            cl_engine_free(engine);
-            return 2;
-        }
-    }
-
-    if ((opt = optget(opts, "pcre-recmatch-limit"))->active) {
-        if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
-            logg("!cli_engine_set_num(CL_ENGINE_PCRE_RECMATCH_LIMIT) failed: %s\n", cl_strerror(ret));
-            cl_engine_free(engine);
-            return 2;
-        }
-    }
-
     if ((opt = optget(opts, "pcre-max-filesize"))->active) {
         if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MAX_FILESIZE, opt->numarg))) {
             logg("!cli_engine_set_num(CL_ENGINE_PCRE_MAX_FILESIZE) failed: %s\n", cl_strerror(ret));
@@ -1022,7 +1011,6 @@ int scanmanager(const struct optstruct *opts)
     /* set scan options */
     if(optget(opts, "allmatch")->enabled) {
         options |= CL_SCAN_ALLMATCHES;
-        cl_engine_set_clcb_virus_found(engine, clamscan_virus_found_cb);
     }
 
     if(optget(opts,"phishing-ssl")->enabled)
@@ -1046,6 +1034,9 @@ int scanmanager(const struct optstruct *opts)
     if(optget(opts, "block-encrypted")->enabled)
         options |= CL_SCAN_BLOCKENCRYPTED;
 
+    if(optget(opts, "block-macros")->enabled)
+        options |= CL_SCAN_BLOCKMACROS;
+
     if(optget(opts, "scan-pe")->enabled)
         options |= CL_SCAN_PE;
 
@@ -1061,15 +1052,25 @@ int scanmanager(const struct optstruct *opts)
     if(optget(opts, "scan-swf")->enabled)
         options |= CL_SCAN_SWF;
 
-    if(optget(opts, "scan-html")->enabled)
+    if(optget(opts, "scan-html")->enabled && optget(opts, "normalize")->enabled)
         options |= CL_SCAN_HTML;
 
     if(optget(opts, "scan-mail")->enabled)
         options |= CL_SCAN_MAIL;
 
+    if(optget(opts, "scan-xmldocs")->enabled)
+        options |= CL_SCAN_XMLDOCS;
+
+    if(optget(opts, "scan-hwp3")->enabled)
+        options |= CL_SCAN_HWP3;
+
     if(optget(opts, "algorithmic-detection")->enabled)
         options |= CL_SCAN_ALGORITHMIC;
 
+    if(optget(opts, "block-max")->enabled) {
+        options |= CL_SCAN_BLOCKMAX;
+    }
+
 #ifdef HAVE__INTERNAL__SHA_COLLECT
     if(optget(opts, "dev-collect-hashes")->enabled)
         options |= CL_SCAN_INTERNAL_COLLECT_SHA;
@@ -1126,11 +1127,6 @@ int scanmanager(const struct optstruct *opts)
         procdev = sb.st_dev;
 #endif
 
-#if HAVE_JSON
-    if (optget(opts, "gen-json")->enabled)
-        options |= CL_SCAN_FILE_PROPERTIES;
-#endif
-
     /* check filetype */
     if(!opts->filename && !optget(opts, "file-list")->enabled) {
         /* we need full path for some reasons (eg. archive handling) */
