@@ -32,7 +32,7 @@ void usage(char *name)
     printf("\n");
     printf("                       Clam AntiVirus: Monitoring Tool %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2008-2018 Cisco Systems, Inc.\n");
+    printf("           (C) 2019 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    %s -hHinpVv?\n", name);
     printf("\n");
@@ -108,6 +108,13 @@ size_t write_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
     return len;
 }
 
+/**
+ * @brief Parse a value from a JSON object, given a key.
+ * 
+ * @param ps_json_obj   The JSON object
+ * @param key           The Key
+ * @return const char*  The Value on Success, NULL on Failure.
+ */
 const char* presigned_get_string(json_object * ps_json_obj, char * key)
 {
     json_object * json_obj = NULL;
@@ -117,18 +124,17 @@ const char* presigned_get_string(json_object * ps_json_obj, char * key)
         json_str = json_object_get_string(json_obj);
         if (json_str == NULL) {
             fprintf(stderr, "Error: json_object_get_string() for %s.\n", key);
-            exit(1);
         }
     } else {
         fprintf(stderr, "Error: json_object_object_get_ex() for %s.\n", key);
-        exit(1);
     }
     return json_str;
 }
 
 int main(int argc, char *argv[])
 {
-    CURL *clam_curl, *aws_curl;
+    int status = 1;
+    CURL *clam_curl = NULL, *aws_curl = NULL;
     CURLcode res;
     int ch;
     struct curl_httppost *post=NULL, *last=NULL;
@@ -146,7 +152,7 @@ int main(int argc, char *argv[])
     char * submissionID = NULL;
     char * fpvname = NULL;
     char *sp, *ep, *str;
-    char * authenticity_token;
+    char * authenticity_token = NULL;
     char * urlp;
 
     curl_global_init(CURL_GLOBAL_ALL);
@@ -154,7 +160,7 @@ int main(int argc, char *argv[])
     clam_curl = curl_easy_init();
     if (clam_curl == NULL) {
         fprintf(stderr, "ERROR: Could not initialize libcurl.\n");
-        exit(1);
+        goto cleanup;
     }
 
     while ((ch = my_getopt(argc, argv, OPTS)) > 0) {
@@ -200,7 +206,7 @@ int main(int argc, char *argv[])
         filename = read_stream();
         if (!(filename)) {
             fprintf(stderr, "ERROR: Unable to read stream\n");
-            exit(1);
+            goto cleanup;
         }
         fromStream=1;
     }
@@ -208,9 +214,9 @@ int main(int argc, char *argv[])
 
     /*** The GET malware|fp ***/
     if (malware == 1)
-        urlp = "http://www.clamav.net/reports/malware";
+        urlp = "https://www.clamav.net/reports/malware";
     else
-        urlp = "http://www.clamav.net/reports/fp";
+        urlp = "https://www.clamav.net/reports/fp";
     curl_easy_setopt(clam_curl, CURLOPT_URL, urlp);
     curl_easy_setopt(clam_curl, CURLOPT_HTTPGET, 1);
     curl_easy_setopt(clam_curl, CURLOPT_WRITEDATA, &wd);
@@ -218,31 +224,31 @@ int main(int argc, char *argv[])
     curl_easy_setopt(clam_curl, CURLOPT_HEADERDATA, &hd_malware);
     curl_easy_setopt(clam_curl, CURLOPT_HEADERFUNCTION, header_cb);
     res = curl_easy_perform(clam_curl);
-    if (res) {
+    if (res != CURLE_OK) {
         fprintf(stderr, "Error in GET %s: %s\n", urlp , curl_easy_strerror(res));
-        exit(1);
+        goto cleanup;
     }
     if (wd.str != NULL) {
         sp = strstr(wd.str, "name=\"authenticity_token\"");
         if (sp == NULL) {
             fprintf (stderr, "Authenticity token element not found.\n");
-            exit(1);
+            goto cleanup;
         }
         sp = strstr(sp, "value=");
         if (sp == NULL) {
             fprintf (stderr, "Authenticity token value not found.\n");
-            exit(1);
+            goto cleanup;
         }
         sp += 7;
         ep = strchr(sp, '"');
         if (ep == NULL) {
             fprintf (stderr, "Authenticity token malformed.\n");
-            exit(1);
+            goto cleanup;
         }
         authenticity_token = malloc(ep-sp+1);
         if (authenticity_token == NULL) {
             fprintf (stderr, "no memory for authenticity token.\n");
-            exit(1);
+            goto cleanup;
         }
         memcpy(authenticity_token, sp, ep-sp);
         authenticity_token[ep-sp] = '\0';
@@ -255,25 +261,26 @@ int main(int argc, char *argv[])
 
     /*** The GET presigned ***/
     if (malware == 1)
-        curl_easy_setopt(clam_curl, CURLOPT_URL, "http://www.clamav.net/presigned?type=malware");
+        curl_easy_setopt(clam_curl, CURLOPT_URL, "https://www.clamav.net/presigned?type=malware");
     else
-        curl_easy_setopt(clam_curl, CURLOPT_URL, "http://www.clamav.net/presigned?type=fp");
+        curl_easy_setopt(clam_curl, CURLOPT_URL, "https://www.clamav.net/presigned?type=fp");
     curl_easy_setopt(clam_curl, CURLOPT_HTTPGET, 1);
 
     if (NULL == hd_malware.cfduid || NULL == hd_malware.session) {
         fprintf (stderr, "invalid cfduid and/or session id values provided by clamav.net/presigned. Unable to continue submission.");
-        exit(1);
+        goto cleanup;
     }
 
     len = strlen(hd_malware.cfduid) + strlen(hd_malware.session) + 3;
     str = malloc(len);
     if (str == NULL) {
         fprintf(stderr, "No memory for GET presigned cookies\n");
-        exit(1);
+        goto cleanup;
     }
     if (snprintf(str, len, "%s; %s;", hd_malware.cfduid, hd_malware.session) > len) {
         fprintf(stderr, "snprintf() failed formatting GET presigned cookies\n");
-        exit(1);
+        free(str);
+        goto cleanup;
     }
     curl_easy_setopt(clam_curl, CURLOPT_COOKIE, str);
     free(str);
@@ -281,11 +288,12 @@ int main(int argc, char *argv[])
     str = malloc(len);
     if (str == NULL) {
         fprintf(stderr, "No memory for GET presigned X-CSRF-Token\n");
-        exit(1);
+        goto cleanup;
     }
     if (snprintf(str, len, "X-CSRF-Token: %s", authenticity_token) > len) {
         fprintf(stderr, "snprintf() failed for GET presigned X-CSRF-Token\n");
-        exit(1);
+        free(str);
+        goto cleanup;
     }
     slist = curl_slist_append(slist, str);
     free(str);
@@ -293,14 +301,14 @@ int main(int argc, char *argv[])
     curl_easy_setopt(clam_curl, CURLOPT_HEADERDATA, &hd_presigned);
     curl_easy_setopt(clam_curl, CURLOPT_HEADERFUNCTION, header_cb);
     if (malware ==1)
-        curl_easy_setopt(clam_curl, CURLOPT_REFERER, "http://www.clamav.net/reports/malware");
+        curl_easy_setopt(clam_curl, CURLOPT_REFERER, "https://www.clamav.net/reports/malware");
     else
-        curl_easy_setopt(clam_curl, CURLOPT_REFERER, "http://www.clamav.net/reports/fp");
+        curl_easy_setopt(clam_curl, CURLOPT_REFERER, "https://www.clamav.net/reports/fp");
 
     res = curl_easy_perform(clam_curl);
-    if (res) {
+    if (res != CURLE_OK) {
         fprintf(stderr, "Error in GET presigned: %s\n", curl_easy_strerror(res));
-        exit(1);
+        goto cleanup;
     }
     curl_slist_free_all(slist);
     slist = NULL;
@@ -310,58 +318,99 @@ int main(int argc, char *argv[])
     ps_json_obj = json_tokener_parse(wd.str);
     if (ps_json_obj == NULL) {
         fprintf(stderr, "Error in json_tokener_parse of %.*s\n", wd.len, wd.str);
-        exit(1);
+        goto cleanup;
     }
     json_str = presigned_get_string(ps_json_obj, "key");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing key from json object\n");
+        goto cleanup;
+    }
     sp = strchr(json_str, '/');
     if (sp == NULL) {
         fprintf(stderr, "Error: malformed 'key' string in GET presigned response (missing '/'.\n");
-        exit (1);
+        goto cleanup;
     }
     sp++;
     ep = strchr(sp, '-');
     if (ep == NULL) {
         fprintf(stderr, "Error: malformed 'key' string in GET presigned response (missing '-'.\n");
-        exit (1);
+        goto cleanup;
     }
     submissionID = malloc(ep-sp+1);
     if (submissionID == NULL) {
         fprintf(stderr, "Error: malloc submissionID.\n");
-        exit (1);
+        goto cleanup;
     }
     memcpy(submissionID, sp, ep-sp);
     submissionID[ep-sp] = '\0';
     aws_curl = curl_easy_init();
     if (!(aws_curl)) {
         fprintf(stderr, "ERROR: Could not initialize libcurl POST presigned\n");
-        exit(1);
+        goto cleanup;
     }
     submissionID[ep-sp] = '\0';
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "key", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "acl");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing acl from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "acl", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "policy");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing policy from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "policy", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "x-amz-meta-original-filename");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing x-amz-meta-original-filename from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "x-amz-meta-original-filename", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "x-amz-credential");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing x-amz-credential from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "x-amz-credential", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "x-amz-algorithm");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing x-amz-algorithm from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "x-amz-algorithm", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "x-amz-date");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing x-amz-date from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "x-amz-date", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     json_str = presigned_get_string(ps_json_obj, "x-amz-signature");
+    if (json_str == NULL) {
+        fprintf(stderr, "Error in presigned_get_string parsing x-amz-signature from json object\n");
+        goto cleanup;
+    }
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "x-amz-signature", CURLFORM_COPYCONTENTS, json_str, CURLFORM_END);
+
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "file", CURLFORM_FILE, filename, CURLFORM_END);
+
     slist = curl_slist_append(slist, "Expect:");
     curl_easy_setopt(aws_curl, CURLOPT_HTTPHEADER, slist);
-    curl_easy_setopt(aws_curl, CURLOPT_URL, "http://clamav-site.s3.amazonaws.com/");
+    curl_easy_setopt(aws_curl, CURLOPT_URL, "https://clamav-site.s3.amazonaws.com/");
     curl_easy_setopt(aws_curl, CURLOPT_HTTPPOST, post);
 
     res = curl_easy_perform(aws_curl);
-    if (res) {
+    if (res != CURLE_OK) {
         fprintf(stderr, "Error in POST AWS: %s\n", curl_easy_strerror(res));
-        exit(1);
+        goto cleanup;
     }
     curl_slist_free_all(slist);
     slist = NULL;
@@ -369,6 +418,7 @@ int main(int argc, char *argv[])
     post = NULL;
     last = NULL;
     curl_easy_cleanup(aws_curl);
+    aws_curl = NULL;
     json_object_put(ps_json_obj);
     free(wd.str);
     wd.str = NULL;
@@ -381,11 +431,12 @@ int main(int argc, char *argv[])
     str = malloc(len);
     if (str == NULL) {
         fprintf(stderr, "No memory for POST submit cookies.\n");
-        exit(1);
+        goto cleanup;
     }
     if (snprintf(str, len, "%s; %s;", hd_malware.cfduid, hd_malware.session) > len) {
         fprintf(stderr, "snprintf() failed formatting POST submit cookies\n");
-        exit(1);
+        free(str);
+        goto cleanup;
     }
     curl_easy_setopt(clam_curl, CURLOPT_COOKIE, str);
     free(str);
@@ -405,13 +456,13 @@ int main(int argc, char *argv[])
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "notify", CURLFORM_COPYCONTENTS, "on", CURLFORM_END);
     curl_formadd(&post, &last, CURLFORM_COPYNAME, "privacy", CURLFORM_COPYCONTENTS, "on", CURLFORM_END);
     curl_easy_setopt(clam_curl, CURLOPT_HTTPHEADER, slist);
-    curl_easy_setopt(clam_curl, CURLOPT_URL, "http://www.clamav.net/reports/submit");
+    curl_easy_setopt(clam_curl, CURLOPT_URL, "https://www.clamav.net/reports/submit");
     curl_easy_setopt(clam_curl, CURLOPT_HTTPPOST, post);
     curl_easy_setopt(clam_curl, CURLOPT_HEADERFUNCTION, NULL);
     res = curl_easy_perform(clam_curl);
-    if (res) {
+    if (res != CURLE_OK) {
         fprintf(stderr, "Error in POST submit: %s\n", curl_easy_strerror(res));
-        exit(1);
+        goto cleanup;
     } else {
         long response_code;
         curl_easy_getinfo(clam_curl, CURLINFO_RESPONSE_CODE, &response_code);
@@ -419,43 +470,75 @@ int main(int argc, char *argv[])
             curl_easy_getinfo(clam_curl, CURLINFO_REDIRECT_URL, &urlp);
             if (urlp == NULL) {
                 fprintf(stderr, "POST submit Location URL is NULL.\n");
-                exit(1);
+                goto cleanup;
             }
             sp = strstr(urlp, "/reports/");
             if (sp == NULL) {
                 fprintf(stderr, "POST submit Location URL is malformed.\n");
-                exit(1);
             }
-            if (!strcmp(sp, "/reports/success"))
+            else if (!strcmp(sp, "/reports/success")) {
                 fprintf(stdout, "Submission success!\n");
-            else if (!strcmp(sp, "/reports/failure"))
+                status = 0;
+            }
+            else if (!strcmp(sp, "/reports/failure")) {
                 fprintf(stdout, "Submission failed\n");
-            else
+            }
+            else {
                 fprintf(stdout, "Unknown submission status %s\n", sp);
+            }
         }
-        else
+        else {
             fprintf(stderr, "Unexpected POST submit response code: %li\n", response_code);
+        }
     }
-    curl_slist_free_all(slist);
-    curl_formfree(post);
-    curl_easy_cleanup(clam_curl);
+
+cleanup:
+    /* 
+     * Cleanup
+     */
+    if (slist != NULL) {
+        curl_slist_free_all(slist);
+    }
+    if (post != NULL) {
+        curl_formfree(post);
+    }
+    if (clam_curl != NULL) {
+        curl_easy_cleanup(clam_curl);
+    }
+    if (aws_curl != NULL) {
+        curl_easy_cleanup(aws_curl);
+    }
+    curl_global_cleanup();
+
     if (wd.str != NULL) {
         free(wd.str);
         wd.str = NULL;
         wd.len = 0;
     }
-    free(hd_malware.cfduid);
-    free(hd_malware.session);
-    free(hd_presigned.cfduid);
-    free(hd_presigned.session);
-    free(submissionID);
-    free(authenticity_token);
-    if (fromStream) {
+    if (hd_malware.cfduid != NULL) {
+        free(hd_malware.cfduid);
+    }
+    if (hd_malware.session != NULL) {
+        free(hd_malware.session);
+    }
+    if (hd_presigned.cfduid != NULL) {
+        free(hd_presigned.cfduid);
+    }
+    if (hd_presigned.session != NULL) {
+        free(hd_presigned.session);
+    }
+    if (submissionID != NULL) {
+        free(submissionID);
+    } 
+    if (authenticity_token != NULL) {
+        free(authenticity_token);
+    }
+    if ((fromStream != 0) && (filename != NULL)) {
         remove(filename);
         free(filename);
     }
 
-    return 0;
+    return status;
 }
 
 
