@@ -115,7 +115,13 @@ char *cl_base64_encode(void *data, size_t len)
     size_t elen;
 
     b64 = BIO_new(BIO_f_base64());
+    if (!(b64))
+        return NULL;
     bio = BIO_new(BIO_s_mem());
+    if (!(bio)) {
+        BIO_free(b64);
+        return NULL;
+    }
 
     bio = BIO_push(b64, bio);
     BIO_write(bio, data, len);
@@ -125,6 +131,10 @@ char *cl_base64_encode(void *data, size_t len)
 
     /* Ensure we're dealing with a NULL-terminated string */
     p = (char *)malloc(elen+1);
+    if (NULL == p) {
+        BIO_free(b64);
+        return NULL;
+    }
     memcpy((void *)p, (void *)buf, elen);
     p[elen] = 0x00;
     buf = p;
