@@ -51,24 +51,33 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <openssl/evp.h>
+
 #if !defined(_WIN32)
 #include <unistd.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "clamav.h"
 #include "default.h"
 #include "others.h"
 #include "libclamav/conv.h"
 #include "libclamav/str.h"
+#include "iowrap.h"
 
 #if defined(_WIN32)
 char * strptime(const char *buf, const char *fmt, struct tm *tm);
 #endif
 
+#if defined(_WIN32)
+#define EXCEPTION_PREAMBLE __try {
+#define EXCEPTION_POSTAMBLE } __except (filter_memcpy(GetExceptionCode(), GetExceptionInformation())) { \
+    winres=1; \
+}
+#else
+#define EXCEPTION_PREAMBLE
+#define EXCEPTION_POSTAMBLE
+#endif
+
 #if !defined(MIN)
     #define MIN(x,y) ((x)<(y)?(x):(y))
 #endif
@@ -136,6 +145,7 @@ unsigned char *cl_hash_data(char *alg, const void *buf, size_t len, unsigned cha
     const EVP_MD *md;
     unsigned int i;
     size_t cur;
+    int winres=0;
 
     md = EVP_get_digestbyname(alg);
     if (!(md))
@@ -155,6 +165,11 @@ unsigned char *cl_hash_data(char *alg, const void *buf, size_t len, unsigned cha
         return NULL;
     }
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_DigestInit_ex(ctx, md, NULL)) {
         if (!(obuf))
             free(ret);
@@ -168,7 +183,9 @@ unsigned char *cl_hash_data(char *alg, const void *buf, size_t len, unsigned cha
 
     cur=0;
     while (cur < len) {
-        size_t todo = MIN(EVP_MD_block_size(md), len-cur);
+        size_t todo = MIN((unsigned long)EVP_MD_block_size(md), (unsigned long)(len-cur));
+
+        EXCEPTION_PREAMBLE
         if (!EVP_DigestUpdate(ctx, (void *)(((unsigned char *)buf)+cur), todo)) {
             if (!(obuf))
                 free(ret);
@@ -179,6 +196,18 @@ unsigned char *cl_hash_data(char *alg, const void *buf, size_t len, unsigned cha
             EVP_MD_CTX_destroy(ctx);
             return NULL;
         }
+        EXCEPTION_POSTAMBLE
+
+        if (winres) {
+            if (!(obuf))
+                free(ret);
+
+            if ((olen))
+                *olen = 0;
+
+            EVP_MD_CTX_destroy(ctx);
+            return NULL;
+        }
 
         cur += todo;
     }
@@ -216,6 +245,11 @@ unsigned char *cl_hash_file_fd(int fd, char *alg, unsigned int *olen)
     if (!(ctx))
         return NULL;
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_DigestInit_ex(ctx, md, NULL)) {
         EVP_MD_CTX_destroy(ctx);
         return NULL;
@@ -234,6 +268,7 @@ unsigned char *cl_hash_file_fd_ctx(EVP_MD_CTX *ctx, int fd, unsigned int *olen)
     int mdsz;
     unsigned int hashlen;
     STATBUF sb;
+    int winres=0;
 
 	unsigned int blocksize;
 
@@ -271,10 +306,19 @@ unsigned char *cl_hash_file_fd_ctx(EVP_MD_CTX *ctx, int fd, unsigned int *olen)
 #else
     while ((nread = read(fd, buf, blocksize)) > 0) {
 #endif
+        EXCEPTION_PREAMBLE
         if (!EVP_DigestUpdate(ctx, buf, nread)) {
             free(buf);
             free(hash);
 
+            return NULL;
+        }
+        EXCEPTION_POSTAMBLE
+
+        if (winres) {
+            free(buf);
+            free(hash);
+
             return NULL;
         }
     }
@@ -325,6 +369,11 @@ int cl_verify_signature_hash(EVP_PKEY *pkey, char *alg, unsigned char *sig, unsi
 
     mdsz = EVP_MD_size(md);
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_VerifyInit_ex(ctx, md, NULL)) {
         EVP_MD_CTX_destroy(ctx);
         return -1;
@@ -369,6 +418,11 @@ int cl_verify_signature_fd(EVP_PKEY *pkey, char *alg, unsigned char *sig, unsign
         return -1;
     }
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_VerifyInit_ex(ctx, md, NULL)) {
         free(digest);
         EVP_MD_CTX_destroy(ctx);
@@ -403,7 +457,7 @@ int cl_verify_signature(EVP_PKEY *pkey, char *alg, unsigned char *sig, unsigned
         unsigned char *newsig;
         size_t newsiglen;
 
-        newsig = (unsigned char *)cl_base64_decode((char *)sig, siglen, NULL, &newsiglen);
+        newsig = (unsigned char *)cl_base64_decode((char *)sig, siglen, NULL, &newsiglen, 1);
         if (!(newsig))
             return -1;
 
@@ -439,6 +493,11 @@ int cl_verify_signature(EVP_PKEY *pkey, char *alg, unsigned char *sig, unsigned
         return -1;
     }
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_VerifyInit_ex(ctx, md, NULL)) {
         free(digest);
         if (decode)
@@ -647,6 +706,11 @@ unsigned char *cl_sign_data(EVP_PKEY *pkey, char *alg, unsigned char *hash, unsi
         return NULL;
     }
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_SignInit_ex(ctx, md, NULL)) {
         free(sig);
         EVP_MD_CTX_destroy(ctx);
@@ -740,7 +804,7 @@ X509 *cl_get_x509_from_mem(void *data, unsigned int len)
 
 int cl_validate_certificate_chain_ts_dir(char *tsdir, char *certpath)
 {
-    char **authorities=NULL, **t, *fullpath;
+    char **authorities=NULL, **t;
     size_t nauths = 0;
     int res;
     DIR *dp;
@@ -968,7 +1032,6 @@ struct tm *cl_ASN1_GetTimeT(ASN1_TIME *timeobj)
 {
     struct tm *t;
     char* str;
-    size_t i = 0;
     const char *fmt=NULL;
     time_t localt;
 #ifdef _WIN32
@@ -1035,8 +1098,6 @@ X509_CRL *cl_load_crl(const char *file)
 {
     X509_CRL *x=NULL;
     FILE *fp;
-    struct tm *tm;
-    time_t crltime;
 
     if (!(file))
         return NULL;
@@ -1050,21 +1111,13 @@ X509_CRL *cl_load_crl(const char *file)
     fclose(fp);
 
     if ((x)) {
-        tm = cl_ASN1_GetTimeT(x->crl->nextUpdate);
-        if (!(tm)) {
-            X509_CRL_free(x);
-            return NULL;
-        }
+	ASN1_TIME *tme;
 
-#if !defined(_WIN32)
-        if (timegm(tm) < time(NULL)) {
-            X509_CRL_free(x);
-            free(tm);
-            return NULL;
-        }
-#endif
-
-        free(tm);
+	tme = X509_CRL_get_nextUpdate(x);
+	if (!tme || X509_cmp_current_time(tme) < 0) {
+		X509_CRL_free(x);
+		return NULL;
+	}
     }
 
     return x;
@@ -1084,6 +1137,11 @@ void *cl_hash_init(const char *alg)
         return NULL;
     }
 
+#ifdef EVP_MD_CTX_FLAG_NON_FIPS_ALLOW
+    /* we will be using MD5, which is not allowed under FIPS */
+    EVP_MD_CTX_set_flags(ctx, EVP_MD_CTX_FLAG_NON_FIPS_ALLOW);
+#endif
+
     if (!EVP_DigestInit_ex(ctx, md, NULL)) {
         EVP_MD_CTX_destroy(ctx);
         return NULL;
@@ -1094,11 +1152,18 @@ void *cl_hash_init(const char *alg)
 
 int cl_update_hash(void *ctx, void *data, size_t sz)
 {
+    int winres=0;
+
     if (!(ctx) || !(data))
         return -1;
 
+    EXCEPTION_PREAMBLE
     if (!EVP_DigestUpdate((EVP_MD_CTX *)ctx, data, sz))
         return -1;
+    EXCEPTION_POSTAMBLE
+
+    if (winres)
+        return -1;
 
     return 0;
 }
