@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014 Cisco Systems, Inc.
+ *  Copyright (C) 2014-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Steven Morgan <smorgan@sourcefire.com>
  *
@@ -229,7 +229,7 @@ int openioc_parse(const char * fname, int fd, struct cl_engine *engine, unsigned
     else
         iocp++;
 
-    ioclen = strlen(fname);
+    ioclen = (uint16_t)strlen(fname);
 
     if (elems != NULL) {
         if (NULL == engine->hm_hdb) {
