@@ -41,7 +41,6 @@
 #include <sys/sysctl.h>
 #endif
 #endif
-#include <dlfcn.h>
 #else
 #include <Windows.h>
 #include <tchar.h>
@@ -279,9 +278,10 @@ end:
 #ifdef CL_THREAD_SAFE
     err = pthread_mutex_unlock(&(intel->mutex));
     if (err) {
-        cli_warnmsg("clamav_stats_add_sample: unlcoking mutex failed (err: %d): %s\n", err, strerror(err));
+        cli_warnmsg("clamav_stats_add_sample: unlocking mutex failed (err: %d): %s\n", err, strerror(err));
     }
 #endif
+    return;
 }
 
 void clamav_stats_flush(struct cl_engine *engine, void *cbdata)
@@ -492,6 +492,7 @@ void clamav_stats_decrement_count(const char *virname, const unsigned char *md5,
         cli_warnmsg("clamav_stats_decrement_count: unlocking mutex failed (err: %d): %s\n", err, strerror(err));
     }
 #endif
+    return;
 }
 
 size_t clamav_stats_get_num(void *cbdata)
